@@ -5,7 +5,9 @@
 #include <X11/Xfuncproto.h>
 
 /* ErrorF is used by xtrans */
-extern _X_EXPORT void ErrorF(const char *f, ...) _X_ATTRIBUTE_PRINTF(1,2);
+extern _X_EXPORT void
+ErrorF(const char *f, ...)
+_X_ATTRIBUTE_PRINTF(1, 2);
 
 #define TRANS_REOPEN
 #define TRANS_SERVER
